@@ -1,6 +1,8 @@
 import os
 import pytest
 
+collect_ignore = ['setup.py']
+
 
 @pytest.fixture
 def fake_bundle_file():
@@ -8,6 +10,12 @@ def fake_bundle_file():
     return os.path.join(cwd, 'test_bundle.json')
 
 
+@pytest.fixture
+def fake_bundle_file_2():
+    cwd = os.path.dirname(os.path.abspath(__file__))
+    return os.path.join(cwd, 'test_bundle_2.json')
+
+
 @pytest.fixture
 def ccoe_bundle():
     cwd = os.path.dirname(os.path.abspath(__file__))
